@@ -25,7 +25,6 @@
 #include <string.h>
 
 #include <glib.h>
-#include <glib/gi18n.h>
 
 #include "uniquemessage.h"
 #include "uniqueinternals.h"
@@ -90,7 +89,7 @@ unique_message_data_new (void)
  *
  * Copies @message_data.
  *
- * Return value: a copy of the passed #UniqueMessageData
+ * Return value: (transfer full): a copy of the passed #UniqueMessageData
  */
 UniqueMessageData *
 unique_message_data_copy (UniqueMessageData *message_data)
@@ -133,7 +132,7 @@ unique_message_data_free (UniqueMessageData *message_data)
 /**
  * unique_message_data_set:
  * @message_data: a #UniqueMessageData
- * @data: binary blob to set, or %NULL
+ * @data: (allow-none): binary blob to set, or %NULL
  * @length: length of @data
  *
  * Sets @data as the payload of @message_data. Any other data is removed
@@ -146,7 +145,7 @@ unique_message_data_free (UniqueMessageData *message_data)
 void
 unique_message_data_set (UniqueMessageData *message_data,
                          const guchar      *data,
-                         gsize              length)
+                         gssize             length)
 {
   g_return_if_fail (message_data != NULL);
 
@@ -168,13 +167,13 @@ unique_message_data_set (UniqueMessageData *message_data,
         message_data->data = (guchar *) g_strdup ("");
     }
 
-  message_data->length = length;
+  message_data->length = length >= 0 ? length : 0;
 }
 
 /**
  * unique_message_data_get:
  * @message_data: a #UniqueMessageData
- * @length: return location for the length of the contents
+ * @length: (out): return location for the length of the contents
  *
  * Retrieves the raw contents of @message_data set using
  * unique_messaget_data_set().
@@ -300,6 +299,9 @@ message_data_get_text_plain (UniqueMessageData *message_data)
   gsize len;
   GError *error = NULL;
 
+  if (message_data->length == 0)
+    return NULL;
+
   str = g_strdup ((gchar *) message_data->data);
   len = message_data->length;
 
@@ -375,7 +377,7 @@ unique_message_data_set_text (UniqueMessageData *message_data,
  *
  * Retrieves the text set using unique_message_data_set_text().
  *
- * Return value: an allocated string.
+ * Return value: (transfer full): a newly-allocated string.
  */
 gchar *
 unique_message_data_get_text (UniqueMessageData *message_data)
@@ -434,7 +436,8 @@ unique_message_data_set_uris (UniqueMessageData  *message_data,
  * Retrieves a %NULL-terminated string vector containing the URIs set with
  * unique_message_data_set_uris().
  *
- * Return value: (array zero-terminated=1) (element-type utf8) (transfer full): an allocated list of URIs. Use g_strfreev() to free it.
+ * Return value: (array zero-terminated=1) (element-type utf8) (transfer full): a newly-allocated,
+ *   %NULL-terminated list of URIs. Use g_strfreev() to free it.
  */
 gchar **
 unique_message_data_get_uris (UniqueMessageData *message_data)
@@ -483,8 +486,9 @@ unique_message_data_set_filename (UniqueMessageData *message_data,
  *
  * Retrieves the filename set with unique_message_data_set_filename().
  *
- * Return value: a newly allocated string containing the filename.
- *   Use g_free() to free the resources used by the returned value.
+ * Return value: (transfer full): a newly allocated string containing the
+ *   filename. Use g_free() to free the resources used by the returned
+ *   value.
  *
  * Since: 1.0.2
  */
@@ -493,7 +497,7 @@ unique_message_data_get_filename (UniqueMessageData *message_data)
 {
   g_return_val_if_fail (message_data != NULL, NULL);
 
-  return g_memdup (message_data->data, message_data->length);
+  return g_memdup (message_data->data, message_data->length + 1);
 }
 
 /**
@@ -504,7 +508,7 @@ unique_message_data_get_filename (UniqueMessageData *message_data)
  * can use gtk_window_set_screen() to move windows or dialogs to the
  * right screen. This field is always set by the Unique library.
  *
- * Return value: a #GdkScreen
+ * Return value: (transfer none): a #GdkScreen
  */
 GdkScreen *
 unique_message_data_get_screen (UniqueMessageData *message_data)
